@@ -59,7 +59,7 @@ static char *mcast[MAX_NODENAMES];
 static char *nodename_env;
 static int num_nodenames;
        int two_node;
-       char *key_filename;
+       char *key_filename=NULL;
 static char *mcast_name;
 static unsigned char votes;
 static unsigned int expected_votes;
