@@ -27,6 +27,7 @@ EC_WAITING_OFF     = 7
 
 TELNET_PATH = "/usr/bin/telnet"
 SSH_PATH    = "/usr/bin/ssh"
+SSL_PATH    = "@FENCEAGENTSLIBDIR@/telnet_ssl"
 
 all_opt = {
 	"help"    : {
@@ -101,19 +102,47 @@ all_opt = {
 		"getopt" : "x",
 		"help" : "-x             Use ssh connection",
 		"order" : 1 },
+	"ssl" : {
+		"getopt" : "z",
+		"help" : "-z             Use ssl connection",
+		"order" : 1 },
 	"port" : {
 		"getopt" : "n:",
-		"help" : "-n <id>        Physical plug number on device",
+		"help" : "-n <id>        Physical plug number on device or name of virtual machine",
 		"order" : 1 },
 	"switch" : {
 		"getopt" : "s:",
 		"help" : "-s <id>        Physical switch number on device",
 		"order" : 1 },
+	"partition" : {
+		"getopt" : "n:",
+		"help" : "-n <id>        Name of the partition",
+		"order" : 1 },
+	"managed" : {
+		"getopt" : "s:",
+		"help" : "-s <id>        Name of the managed system",
+		"order" : 1 },
 	"test" : {
 		"getopt" : "T",
 		"help" : "",
 		"order" : 1,
-		"obsolete" : "use -o status instead" }
+		"obsolete" : "use -o status instead" },
+	"vmipaddr" : {
+		"getopt" : "A:",
+		"help" : "-A <ip>        IP address or hostname of managed VMware ESX (default localhost)",
+		"order" : 2 },
+	"vmlogin" : {
+		"getopt" : "L:",
+		"help" : "-L <name>      VMware ESX management login name",
+		"order" : 2 },
+	"vmpasswd" : {
+		"getopt" : "P:",
+		"help" : "-P <password>  VMware ESX management login password",
+		"order" : 2 },
+	"vmpasswd_script" : {
+		"getopt" : "B:",
+		"help" : "-B <script>    Script to run to retrieve VMware ESX management password",
+		"order" : 2 }
 }
 
 class fspawn(pexpect.spawn):
@@ -292,6 +321,20 @@ def check_input(device_opt, opt):
 	if options.has_key("-v") and options.has_key("debug_fh") == 0:
 		options["debug_fh"] = sys.stderr
 
+	## VMware
+	#######
+	if options.has_key("-B"):
+		options["-P"] = os.popen(options["-B"]).read().rstrip()
+
+	if (device_opt.count("vmlogin") and (not options.has_key("-L"))):
+		fail_usage("Failed: You have to set login name for VMware ESX management console")
+
+	if (options.has_key("-L") and (not (options.has_key("-P") or options.has_key("-C")))):
+		fail_usage("Failed: You have to enter password or password script for VMware ESX management console")
+
+	if (options.has_key("-L") and (not (options.has_key("-n")))):
+		fail_usage("Failed: You have to enter virtual machine name")
+		
 	return options
 	
 def wait_power_status(tn, options, get_power_fn):
@@ -342,15 +385,30 @@ def fence_login(options):
 		re_login = re.compile("(login: )|(Login Name:  )|(username: )|(User Name :)", re.IGNORECASE)
 		re_pass  = re.compile("password", re.IGNORECASE)
 
-		if options.has_key("-x") and 0 == options.has_key("-k"):
+		if options.has_key("-z"):
+			command = '%s %s %s' % (SSL_PATH, options["-a"], "443")
+			conn = fspawn(command)
+		elif options.has_key("-x") and 0 == options.has_key("-k"):
 			command = '%s %s@%s' % (SSH_PATH, options["-l"], options["-a"])
 			if options.has_key("ssh_options"):
 				command += ' ' + options["ssh_options"]
 			conn = fspawn(command)
-			result = conn.log_expect(options, [ "ssword:", "Are you sure you want to continue connecting (yes/no)?" ], LOGIN_TIMEOUT)
-			if result == 1:
-				conn.sendline("yes")
-				conn.log_expect(options, "ssword:", LOGIN_TIMEOUT)
+
+			if options.has_key("telnet_over_ssh"):
+				#This is for stupid ssh servers (like ALOM) which behave more like telnet (ignore name and display login prompt)
+				result = conn.log_expect(options, [ re_login, "Are you sure you want to continue connecting (yes/no)?" ], LOGIN_TIMEOUT)
+				if result == 1:
+					conn.sendline("yes") # Host identity confirm
+					conn.log_expect(options, re_login, LOGIN_TIMEOUT)
+
+				conn.sendline(options["-l"])
+				conn.log_expect(options, re_pass, LOGIN_TIMEOUT)
+			else:
+				result = conn.log_expect(options, [ "ssword:", "Are you sure you want to continue connecting (yes/no)?" ], LOGIN_TIMEOUT)
+				if result == 1:
+					conn.sendline("yes")
+					conn.log_expect(options, "ssword:", LOGIN_TIMEOUT)
+
 			conn.sendline(options["-p"])
 			conn.log_expect(options, options["-c"], LOGIN_TIMEOUT)
 		elif options.has_key("-x") and 1 == options.has_key("-k"):
