@@ -40,6 +40,7 @@
 #define DEFAULT_POST_JOIN_DELAY	6
 #define DEFAULT_POST_FAIL_DELAY	0
 #define DEFAULT_CLEAN_START	0
+#define DEFAULT_SKIP_UNDEFINED	0
 #define DEFAULT_OVERRIDE_PATH	"/var/run/cluster/fenced_override"
 #define FENCED_SOCK_PATH	"fenced_socket"
 
@@ -123,9 +124,11 @@ struct commandline
 	int pad;
 	char *override_path;
 	int8_t clean_start;
+	int8_t skip_undefined;
 	int8_t post_join_delay_opt;
 	int8_t post_fail_delay_opt;
 	int8_t clean_start_opt;
+	int8_t skip_undefined_opt;
 	int8_t override_path_opt;
 	int8_t override_time_opt;
 };
@@ -167,6 +170,7 @@ void do_recovery_done(fd_t *fd);
 
 /* agent.c */
 int dispatch_fence_agent(char *victim, int force);
+int count_methods(int cd, char *victim);
 
 /* group.c */
 int setup_groupd(void);
