@@ -1,9 +1,9 @@
 #ifndef __GFS_DOT_H__
 #define __GFS_DOT_H__
 
-#define RELEASE_VERSION "2.03.07"
+#define RELEASE_VERSION "2.03.08"
 
-#include <linux/lm_interface.h>
+#include "lm_interface.h"
 
 #include "gfs_ondisk.h"
 #include "fixed_div64.h"
