@@ -259,17 +259,33 @@ static void are_you_sure(struct gfs2_sbd *sdp)
 {
 	char input[32];
 	struct volume_id *vid = NULL;
+	int fd;
 
-	vid = volume_id_open_node(sdp->device_name);
-	if (vid == NULL)
+	fd = open(sdp->device_name, O_RDONLY);
+	if (fd < 0)
+		die("Error: device %s not found.\n", sdp->device_name);
+	vid = volume_id_open_fd(fd);
+	if (vid == NULL) {
+		close(fd);
 		die("error identifying the contents of %s: %s\n",
 		    sdp->device_name, strerror(errno));
-
+	}
 	printf("This will destroy any data on %s.\n", sdp->device_name);
-	if (volume_id_probe_all(vid, 0, sdp->device_size) == 0)
-		printf("  It appears to contain a %s %s.\n", vid->type,
-			   vid->usage_id == VOLUME_ID_OTHER? "partition" : vid->usage);
+	if (volume_id_probe_all(vid, 0, sdp->device_size) == 0) {
+		const char *fstype, *fsusage;
+		int rc;
+
+		rc = volume_id_get_type(vid, &fstype);
+		if (rc) {
+			rc = volume_id_get_usage(vid, &fsusage);
+			if (!rc || strncmp(fsusage, "other", 5) == 0)
+				fsusage = "partition";
+			printf("  It appears to contain a %s %s.\n", fstype,
+			       fsusage);
+		}
+	}
 	volume_id_close(vid);
+	close(fd);
 	printf("\nAre you sure you want to proceed? [y/n] ");
 	fgets(input, 32, stdin);
 
