@@ -12,6 +12,7 @@
 #include <reslist.h>
 #include <assert.h>
 #include <event.h>
+#include <sets.h>
 
 /* Use address field in this because we never use it internally,
    and there is no extra space in the cman_node_t type.
@@ -39,7 +40,9 @@ pthread_rwlock_t resource_lock = PTHREAD_RWLOCK_INITIALIZER;
 
 void res_build_name(char *, size_t, resource_t *);
 int get_rg_state_local(char *, rg_state_t *);
-int group_migratory(char *, int);
+int group_migratory(char *groupname, int lock);
+int _group_property(char *groupname, char *property, char *ret, size_t len);
+int restart_threshold_exceeded(restart_counter_t arg);
 
 
 struct status_arg {
@@ -70,23 +73,9 @@ node_should_start_safe(uint32_t nodeid, cluster_member_list_t *membership,
 int
 node_domain_set_safe(char *domainname, int **ret, int *retlen, int *flags)
 {
-	fod_t *fod;
-	int rv = -1, found = 0, x = 0;
-
+	int rv = 0;
 	pthread_rwlock_rdlock(&resource_lock);
-
-	list_for(&_domains, fod, x) {
-		if (!strcasecmp(fod->fd_name, domainname)) {
-			found = 1;
-			break;
-		}
-	} // while (!list_done(&_domains, fod));
-
-	if (found) {
-		rv = node_domain_set(fod, ret, retlen);
-		*flags = fod->fd_flags;
-	}
-
+	rv = node_domain_set(&_domains, domainname, ret, retlen, flags);
 	pthread_rwlock_unlock(&resource_lock);
 
 	return rv;
@@ -240,7 +229,7 @@ count_resource_groups_local(cman_node_t *mp)
 		     st.rs_state != RG_STATE_STARTING)
 			continue;
 
-		if (mp->cn_nodeid != st.rs_owner)
+		if (mp->cn_nodeid != (int)st.rs_owner)
 			continue;
 
 		++mp->cn_svccount;
@@ -420,6 +409,47 @@ check_depend_safe(char *rg_name)
 }
 
 
+int
+check_rdomain_crash(char *svcName)
+{
+	int *nodes = NULL, nodecount = 0;
+	int *fd_nodes = NULL, fd_nodecount = 0, fl = 0;
+	int *isect = NULL, icount = 0;
+	char fd_name[256];
+
+	if (_group_property(svcName, "domain", fd_name, sizeof(fd_name)) != 0)
+		goto out_free;
+
+	if (node_domain_set(&_domains, fd_name, &fd_nodes,
+			    &fd_nodecount, &fl) != 0)
+		goto out_free;
+
+	if (!(fl & FOD_RESTRICTED))
+		goto out_free;
+	
+	if (s_intersection(fd_nodes, fd_nodecount, nodes, nodecount, 
+		    &isect, &icount) < 0)
+		goto out_free;
+
+	if (icount == 0) {
+		clulog(LOG_NOTICE, "Marking %s as stopped: "
+		       "Restricted domain unavailable\n", svcName);
+		rt_enqueue_request(svcName, RG_STOP, NULL, 0, 0,
+				   0, 0);
+	}
+
+out_free:
+	if (fd_nodes)
+		free(fd_nodes);
+	if (nodes)
+		free(nodes);
+	if (isect)
+		free(isect);
+
+	return 0;
+}
+
+
 /**
   Start or failback a resource group: if it's not running, start it.
   If it is running and we're a better member to run it, then ask for
@@ -433,6 +463,7 @@ consider_start(resource_node_t *node, char *svcName, rg_state_t *svcStatus,
 	cman_node_t *mp;
 	int autostart, exclusive;
 	struct dlm_lksb lockp;
+	int fod_ret;
 
 	mp = memb_id_to_p(membership, my_id());
 	assert(mp);
@@ -445,7 +476,7 @@ consider_start(resource_node_t *node, char *svcName, rg_state_t *svcStatus,
 	 * local start.
 	 */
 	if (svcStatus->rs_state == RG_STATE_STARTED &&
-	    svcStatus->rs_owner == mp->cn_nodeid)
+	    svcStatus->rs_owner == (uint32_t)mp->cn_nodeid)
 		return;
 
 	if (svcStatus->rs_state == RG_STATE_DISABLED)
@@ -527,10 +558,13 @@ consider_start(resource_node_t *node, char *svcName, rg_state_t *svcStatus,
 	 * Start any stopped services, or started services
 	 * that are owned by a down node.
 	 */
-	if (node_should_start(mp->cn_nodeid, membership, svcName, &_domains) ==
-	    FOD_BEST)
+	fod_ret = node_should_start(mp->cn_nodeid, membership,
+				    svcName, &_domains);
+	if (fod_ret == FOD_BEST)
 		rt_enqueue_request(svcName, RG_START, NULL, 0, mp->cn_nodeid,
 				   0, 0);
+	else if (fod_ret == FOD_ILLEGAL)
+		check_rdomain_crash(svcName);
 }
 
 
@@ -546,7 +580,7 @@ consider_relocate(char *svcName, rg_state_t *svcStatus, uint32_t nodeid,
 	 */
 	if ((svcStatus->rs_state != RG_STATE_STARTING &&
 	    svcStatus->rs_state != RG_STATE_STARTED) ||
-	    svcStatus->rs_owner != my_id())
+	    svcStatus->rs_owner != (uint32_t)my_id())
 		return;
 
 	/*
@@ -750,7 +784,9 @@ eval_groups(int local, uint32_t nodeid, int nodeStatus)
  * @see			eval_groups
  */
 int
-group_event(char *rg_name, uint32_t state, int owner)
+group_event(char __attribute__ ((unused)) *rg_name,
+	    uint32_t state,
+	    int __attribute__ ((unused)) owner)
 {
 	char svcName[64], *nodeName;
 	resource_node_t *node;
@@ -825,7 +861,7 @@ group_event(char *rg_name, uint32_t state, int owner)
 		 */
 		if (depend == 0 &&
 		    svcStatus.rs_state == RG_STATE_STARTED &&
-		    svcStatus.rs_owner == my_id()) {
+		    svcStatus.rs_owner == (uint32_t)my_id()) {
 
 			clulog(LOG_WARNING, "Stopping service %s: Dependency missing\n",
 			       svcName);
@@ -1045,15 +1081,13 @@ out:
    @return		0 on success, -1 on failure.
  */
 int
-group_property(char *groupname, char *property, char *ret, size_t len)
+_group_property(char *groupname, char *property, char *ret, size_t len)
 {
 	resource_t *res = NULL;
 	int x = 0;
 
-	pthread_rwlock_rdlock(&resource_lock);
 	res = find_root_by_ref(&_resources, groupname);
 	if (!res) {
-		pthread_rwlock_unlock(&resource_lock);
 		return -1;
 	}
 
@@ -1061,15 +1095,24 @@ group_property(char *groupname, char *property, char *ret, size_t len)
 		if (strcasecmp(res->r_attrs[x].ra_name, property))
 			continue;
 		strncpy(ret, res->r_attrs[x].ra_value, len);
-		pthread_rwlock_unlock(&resource_lock);
 		return 0;
 	}
-	pthread_rwlock_unlock(&resource_lock);
 
 	return -1;
 }
 
 
+int
+group_property(char *groupname, char *property, char *ret_val, size_t len)
+{
+	int ret = -1;
+	pthread_rwlock_rdlock(&resource_lock);
+	ret = _group_property(groupname, property, ret_val, len);
+	pthread_rwlock_unlock(&resource_lock);
+	return ret;
+}
+	
+
 /**
   Send the state of a resource group to a given file descriptor.
 
@@ -1247,7 +1290,7 @@ rg_doall(int request, int block, char *debugfmt)
 
 			/* Always run stop if we're the owner, regardless
 			   of state; otherwise, don't run stop */
-			if (svcblk.rs_owner != my_id())
+			if (svcblk.rs_owner != (uint32_t)my_id())
 				continue;
 		}
 
@@ -1297,7 +1340,7 @@ q_status_checks(void __attribute__ ((unused)) *arg)
 			rg_unlock(&lockp);
 		}
 
-		if (svcblk.rs_owner != my_id() ||
+		if (svcblk.rs_owner != (uint32_t)my_id() ||
 		    (svcblk.rs_state != RG_STATE_STARTED &&
 		     svcblk.rs_state != RG_STATE_MIGRATE))
 			continue;
@@ -1355,7 +1398,7 @@ do_condstops(void)
 			continue;
 		}
 
-		if (svcblk.rs_owner != my_id())
+		if (svcblk.rs_owner != (uint32_t)my_id())
 			continue;
 
 		/* Set state to uninitialized if we're killing a RG */
@@ -1435,7 +1478,7 @@ do_condstarts(void)
 			rg_unlock(&lockp);
 		}
 
-		if (!need_init && svcblk.rs_owner != my_id())
+		if (!need_init && svcblk.rs_owner != (uint32_t)my_id())
 			continue;
 
 		if (need_init) {
@@ -1558,7 +1601,7 @@ dump_config_version(FILE *fp)
   resource group modification.
  */
 int
-init_resource_groups(int reconfigure)
+init_resource_groups(int reconfigure, int do_init)
 {
 	int fd, x, y, cnt;
 
@@ -1685,10 +1728,14 @@ init_resource_groups(int reconfigure)
 		clulog(LOG_INFO, "Restarting changed resources.\n");
 		do_condstarts();
 	} else {
-		/* Do initial stop-before-start */
-		clulog(LOG_INFO, "Initializing Services\n");
-		rg_doall(RG_INIT, 1, "Initializing %s\n");
-		clulog(LOG_INFO, "Services Initialized\n");
+		if (do_init) {
+			/* Do initial stop-before-start */
+			clulog(LOG_INFO, "Initializing Services\n");
+			rg_doall(RG_INIT, 1, "Initializing %s\n");
+			clulog(LOG_INFO, "Services Initialized\n");
+		} else {
+			clulog(LOG_INFO, "Skipping stop-before-start: overridden by administrator\n");
+		}
 		rg_set_initialized();
 	}
 
